var Vast2Parser = function(loadXmlFunction){

  var self = {}

  function appendTracking(ad, element, value) {
    if (ad.trackings == null) {
      ad.trackings = [];
    }

    var tracking = {
      key: element,
      url: value
    }
    ad.trackings.push(tracking);
  }

  function openStreamWithAd(ad, url, callback) {
    loadXmlFunction(url, function(data) {
        if (data == null) {
          callback(null);
        } else {
          self.parseDom(data, callback, ad);
        }
    });
  }

  function addTrackingToAd(dom, ad) {
    var impressions = XmlUtils.getElement(dom, "Impression");
    if (impressions != null && impressions.length > 0) {
      for (var i = 0; i < impressions.length; i++) {
        url = XmlUtils.getText(impressions.item(i));
        appendTracking(ad, "impressions", url);
      }
    }

    var errors = XmlUtils.getElement(dom, "Error");
    if (errors != null && errors.length > 0){
      for (var i = 0; i < errors.length; i++) {
        url = XmlUtils.getText(errors.item(i));
        appendTracking(ad, "errors", url);
      }
    }

    var trackings = XmlUtils.getElement(dom, "Creatives/Creative/Linear/TrackingEvents/Tracking");
    if (trackings != null && trackings.length > 0) {
      for (var i = 0; i < trackings.length; i++) {
        tracking = trackings.item(i);
        var event = tracking.getAttribute("event");

        if (event == "start") {
          appendTracking(ad, "track_0", XmlUtils.getText(tracking));
        } else if (event == "firstQuartile") {
          Putpat.debug(XmlUtils.getText(url));
          appendTracking(ad, "track_25", XmlUtils.getText(tracking));
        } else if (event == "midpoint") {
          appendTracking(ad, "track_50", XmlUtils.getText(tracking));
        } else if (event == "thirdQuartile") {
          appendTracking(ad, "track_75", XmlUtils.getText(tracking));
        } else if (event == "complete") {
          appendTracking(ad, "track_100", XmlUtils.getText(tracking));
        } else if (event == "fullscreen") {
          appendTracking(ad, "track_fullscreen", XmlUtils.getText(tracking));
        }
      }
    }

    var videoClicks = XmlUtils.getElement(dom, "Creatives/Creative/Linear/VideoClicks/ClickThrough");
    if (videoClicks != null && videoClicks.length > 0) {
      appendTracking(ad, "click_through", XmlUtils.getText(videoClicks.item(0)));
    }

    var clickTracking = XmlUtils.getElement(dom, "Creatives/Creative/Linear/VideoClicks/ClickTracking");
    if (clickTracking != null && clickTracking.length > 0) {
      for (var v = 0; v < clickTracking.length; v++) {
        appendTracking(ad, "click_tracking", XmlUtils.getText(clickTracking.item(v)));
      }
    }
  }

  function addVideosToAd(dom, ad) {
    var videos = XmlUtils.getElement(dom, "Creatives/Creative/Linear");
    if (videos != null && videos.length > 0){
      for (var i = 0; i < videos.length; i++) {
        var mediaFiles = XmlUtils.getElement(videos.item(i), "MediaFiles/MediaFile");

        if (mediaFiles != null && mediaFiles.length > 0) {
          for (var v = 0; v < mediaFiles.length; v++) {
            var mediaFile = mediaFiles.item(v); 
            url = XmlUtils.getText(mediaFile);

            var data = {
              "delivery" : mediaFile.getAttribute("delivery"),
              "bitrate" : mediaFile.getAttribute("bitrate"),
              "width" : mediaFile.getAttribute("width"),
              "height" : mediaFile.getAttribute("height"),
              "type" : mediaFile.getAttribute("type"),
              "url" : url
            }

            if (ad["url"] == null) {
              ad["url"] = [];
            }
            ad["url"].push(data);
          }
        }

        var durations = XmlUtils.getElement(videos.item(i), "Duration");
        if (durations != null && durations.length > 0) {
          var durationText = XmlUtils.getText(durations[0]).trim();
          var match = durationText.match(/^(\d\d):(\d\d):(\d\d)$/);
          if (match) {
            var durationInt = parseInt(match[1], 10) * 60*60 + parseInt(match[2], 10) * 60 + parseInt(match[3], 10);
            ad["duration"] = durationInt;
          }
          match = durationText.match(/^(\d*)$/);
          if (match) {
            ad["duration"] = parseInt(durationText,10);
          }
        }

      }
    }
  }

  function parseWrapper (dom, ad, callback) {
    addTrackingToAd(dom, ad);
    openStreamWithAd(ad, XmlUtils.getText(XmlUtils.getElement(dom, "VASTAdTagURI")), callback);
  }


  function parseInLine (dom, ad, callback) {
    addVideosToAd(dom, ad);
    addTrackingToAd(dom, ad);
    callback(ad);
  }

  self.parseDom = function( dom, callback, ad ) {
    var wrapper = XmlUtils.getElement(dom, "Ad/Wrapper");
    var inline = XmlUtils.getElement(dom, "Ad/InLine");

    if (ad == null || ad == undefined) {
      ad = {}
    }

    Putpat.debug(dom.nodeName);
    Putpat.debug(dom.firstChild.nodeName);

    if (wrapper != null && wrapper.length > 0) {
      Putpat.debug("parsing Wrapper");
      parseWrapper(wrapper, ad, callback);
    } else if (inline != null && inline.length > 0) {
      Putpat.debug("parsing Inline");
      parseInLine(inline, ad, callback);
    } else {
      Putpat.debug("No AD in DOM");
      callback(null);
    }
  }

  return self;
};
