/**
 * Das hier ist die Putpat-Kern-Klasse.
 * Sämtliche Kommunikation mit dem Backend von
 * Putpat passiert hier rüber. Nicht schön.
 * Aber funktioniert. Meistens. :)
 *
 * Unten ist ein Return Block mit allen Methoden die
 * aufgerufen werden können
 */
var Putpat = function() {
    // Versionnummer, wichtig für den Release!
    var VERSION = "2.429";
    // Server, der für die Kommunikation benutzt wird.
    // Hier alternativ die lokale IP oder sowas angeben
    var base_server = "http://www.putpat.tv";
    // Die Webservice Base Methoden
    var base_url_json = base_server + "/ws.json";
    var base_url_xml  = base_server + "/ws.xml";

    // Die neue Player ID
    var playerId = null;
    // Das csrf Tokenk
    var token = null;
    // Der Cookie für die aktuelle Session
    var cookie = null;

    // Details des Nutzers
    var user = null;
    // Hat das anmelden geklappt?
    var loginFailed = false;
    // Zeit versatz zum Server
    var timeDifference = 0;
    // Die Veequalizer des Nutzers
    var veequalizers = null;

    // Die Kanäle von Putpat
    var putpat_channels = null;

    // Das Default User-Logo, wird beim Anmelden ersetzt durch die 
    // korrekte Version
    var userCornerLogo = "http://files.putpat.tv/samsung/dein_kanal.png";

    // Bilder zum Error-Reporting (Cross-Domain)
    var reportErrorImages = [];

    /**
     * Die Init-Methode wird beim Start von Putpat aufgerufen. Hier wird
     * der CSRF Token geholt
     */
    function init(callback) {
        if (token == null) {
            callAPI(base_url_json + "?method=User.token", "GET", null, false, function(result) {
                Putpat.debug("Token bekommen");
                if (result != null) {
                  try {
                    json = JSON.parse(result);
                    token = json["session"]["token"];
                    callback(true);
                  } catch(e) {
                    Putpat.debug("Fehler beim Token fetchen!");
                  }
                } else {
                    callback(false);
                }
            });
        } else {
            callback(true);
        }
    }

    /**
     * Diese Funktion übernimmt sämtliche aufrufe an Putpat. 
     * url       - Die URL, die aufgerufen werden soll.
     * method    - Post, Get, ...
     * data      - Payload für Post Aufrufe
     * anonymous - Soll der Aufruf ohne Cookie und ohne User Session sein?
     *             Das ist Notwendig, wenn eine Nicht-Putpat-URL aufgerufen wird.
     * format    - JSON, XML, ...
     * callback  - Diese Methode wird mit dem Ergebnis aufgerufen. Null, wenn
     *             es kein erfolgreiches 200 gab
     * openErrorDialog - Soll der Fehlerdialog geöffnet werden, wenn der Call
     *             fehlgeschlagen ist? Oder lieber stillschweigend ignorieren?
     * addClient - Soll der Client automatisch hinzugefügt werden
     */
    function callAPIwithFormat(url, method, data, anonymous, format, callback,
                               openErrorDialog, addClient) {
        if (Utils.isStringEmpty(url)) {
          Putpat.debug("ERROR : EMPTY URL");
          callback(null);
          return;
        }

        if (openErrorDialog == null) {
          openErrorDialog = true;
        }

        if (addClient || addClient == null) {
            url = url + "&client="+CLIENT_STRING+"&player_id=" + playerId;
        }

        // Cookie setzen, wenn nicht anonym
        if (token != null && !anonymous) {
            Putpat.debug("Not anonymous! " + token);

            if (cookie == null || cookie == "") {
              cookie = "_csrf_token=" + token;
            } else {
              cookie = cookie + ";_csrf_token=" + token;
            }
        }

        XHRPool.getXHR(function(xhr) {
            xhr.onreadystatechange = function() {
              try {
                if(xhr.readyState == 4 && xhr.status == 200) {
                  if (!Device.browserCookies && !anonymous) {
                    Putpat.debug("Cookie was : " + cookie);
                    var newcookie = xhr.getResponseHeader( "Set-Cookie" );
                    if (newcookie != null) {
                      cookie = newcookie;
                      Putpat.debug("Cookie set to " + cookie);
                    }
                  }
                  if (callback) {
                     if (format == "xml") {
                       var xmlData = xhr.responseXML;
                       XHRPool.done(xhr);
                       callback(xmlData);
                     } else {
                       var textData = xhr.responseText;
                       XHRPool.done(xhr);
                       callback(textData);
                     }
                  } else {
                    XHRPool.done(xhr);
                  }
                } else if (openErrorDialog && xhr.readyState == 4 && 
                           xhr.status != 200 && xhr.status != 0) {
                  XHRPool.done(xhr);
                  ErrorView.show("error", "Es konnte keine Internetverbindung hergestellt werden. "+
                                 "Bitte überprüfe Dein Netzwerk.");
                  Putpat.debug("FEHLER !!!" + xhr.responseText + " - " + xhr.status);
                } else if (xhr.readyState == 4) {
                  XHRPool.done(xhr);
                  callback(null);
                }
              } catch (e) {
                Putpat.debug("error 101");
                if (callback != null && callback != undefined) {
                  callback(null);
                }
              }
            };

            Putpat.debug(url);
            xhr.open(method, url);

            if (!Device.browserCookies && cookie != null && !anonymous) {
              xhr.withCredentials = true;
              xhr.setRequestHeader( "Cookie", cookie);
            }
            Putpat.debug(method + " ... " + url);

            if (format != null) {
                xhr.setRequestHeader("Content-Type", "text/"+format);
            }

            xhr.send(data);
        });
    }

    function callAPI(url, method, data, anonymous, callback, openErrorDialog, addClient) {
        callAPIwithFormat(url, method, data, anonymous, "json", callback, 
                          openErrorDialog, addClient);
    }

    function updateUser(currentUser, result) {
        var dataChanged = false;
        if (currentUser["first_name"] != result["first_name"]) {
            currentUser["first_name"] = result["first_name"];
            dataChanged = true;
        }

        var isSamsung = false;

        for (var i in result["group_infos"]) {
            var group = result["group_infos"][i];
            if (group["name"] == "samsung") {
                isSamsung = true;
            }
        }

        if (currentUser["real_account"] == null || currentUser["real_account"] != !isSamsung){
            currentUser["real_account"] = !isSamsung;
            dataChanged = true;
        }

        if (dataChanged) {
            Settings.updateUser(currentUser);
        }
    }

    function parseLoginResult(currentUser, result, callback) {
       if (result != null) {
           json = JSON.parse(result);


           if (json["error"] != null) {
               loginFailed = true;
               callback(!loginFailed, json["error"]["message"]);
           } else {
               user = json["user"];
               if (user["cornerlogo"] != null && user["cornerlogo"] != undefined) {
                 userCornerLogo = user["cornerlogo"];
               }
               updateUser(currentUser, user);
               loginFailed = false;
               doFetchTimeDifference(callback);
           }
       } else {
           loginFailed = true;
           callback(!loginFailed);
       }
     }

    function doLogin(currentUser, callback) {
       Putpat.veequalizers = null;
       Channels.resetCurrentVq();
       // User existiert schon
       if (currentUser != null) {
         Putpat.debug("LOGIN USER");
         if (currentUser["type"] == "LOGIN") {
            callAPI(base_url_json + "?method=User.login" + 
                    "&username=" + encodeURIComponent(currentUser["login"]) +
                    "&password=" + encodeURIComponent(currentUser["password"]),
                    "POST", " ", false, function (result) {
                parseLoginResult(currentUser, result, callback);
            });
         } else if (currentUser["type"] == "UUID") {
            callAPI(base_url_json + "?method=User.login_with_unique"+
                    "&uuid=" + encodeURIComponent(currentUser["uuid"]) +
                    "&token=" + encodeURIComponent(currentUser["token"]),
                    "POST", " ", false, function (result) {
                parseLoginResult(currentUser,result, callback);
            });
         } else if (currentUser["type"] == "DEVICE_LOGIN") {
            callAPI(base_url_json + "?method=DeviceLogin.login_with_unique"+
                    "&password=" + encodeURIComponent(currentUser["password"]) +
                    "&email=" + encodeURIComponent(currentUser["login"]),
                    "POST", " ", false, function (result) {
                parseLoginResult(currentUser, result, callback);
            });
         }
       } else { // User existiert noch nicht
        var uuid = Utils.createUUID();

        Putpat.debug("Noch kein User!!");
        callAPI(base_url_json + "?method=User.create_with_unique&uuid=" + encodeURIComponent(uuid) +
                "&first_name=Putpat&group=samsung",
                "POST", " ", false, function (result) {

            if (result != null) {
                json = JSON.parse(result);

                if (json["error"] != null) {
                    loginFailed = true;
                    callback(!loginFailed);
                } else {
                    token = json["user"]["token"];
                    loginFailed = false;

                    var newuser = {
                        "type": "UUID",
                        "uuid": uuid,
                        "token": token,
                        "real_account": false
                    };

                    Settings.setCurrentUser(newuser);
                    callAPI(base_url_json + "?method=User.current", "GET", null, false, function (result) {
                        json = JSON.parse(result);
                        user = json["user"];
                        doFetchTimeDifference(callback);
                    });

                }
            } else {
                loginFailed = true;
                callback(!loginFailed);
            }
        });

      }
    }

    function doFetchTimeDifference(callback) {
        callAPI(base_url_json + "?method=Channel.time", "GET", null, true, function(result) {
            result = JSON.parse(result);
            time = result["time"]["milliseconds"];
            Putpat.timeDifference = new Date().getTime() - time;
            callback(true);
        });
    }

    function doFetchVeequalizer(callback) {
        callAPI(base_url_json+ "?method=User.veequalizerPresets", "GET", null, false, function(result) {
            json = JSON.parse(result);
            Putpat.veequalizers = new Array();

            for (i in json) {
                Putpat.veequalizers.push(json[i]["veequalizer_preset"]);
            }

            if (callback) {
                callback(Putpat.veequalizers);
            }
        });
    }

    function doFetchChannels(callback) {
        if (putpat_channels == null) {
            Putpat.debug("Loading channels");
            callAPI(base_url_json + "?method=Channel.all", "GET", null, false, function(result) {
               try {
                result_channels = JSON.parse(result);

                channels = new Array();

                if (user == null) { user = {}; }
                if (user["channel_name"] == null) { user["channel_name"] = "Mein Channel";}

                channels.push({
                    title: user["channel_name"],
                    id: 0,
                    channel_message: "Den hab ich mir selbst gemacht",
                    channel_info: "Hallo,\n\n"+
                            "willkommen zu Hause. Dies ist Dein ganz persönlicher Musik-Kanal, Dein Musikfernsehen.\n"+
                            "Hier bist Du Programmdirektor und Macher. Deine Lieblingskünstler, Deine Musikrichtung,\n"+
                            "Deine Stimmung.\n\n"+
                            "Aus tausenden von Musikvideos, die Putpat hat, laufen hier Deine Clips - Deine Musik, rund "+
                            "um die Uhr und ohne Ende. Gib dem Veequalizer die Sporen und entdecke immer wieder neu, probiere,\n"+
                            "sei neugierig.",
                    sort_id: 0,
                    color: "#00b4ff",
                    disable_advertisement: false
                });

                for (var i in result_channels) {
                    if (result_channels[i]["channel"]["on_air"] == true) {
                        channels.push(result_channels[i]["channel"]);
                    }
                }

                channels.sort(function(a, b) {
                    if (a.sort_id > b.sort_id)
                        return 1;
                    else if (a.sort_id < b.sort_id)
                        return -1;

                    return 0;
                });


                putpat_channels = channels;
                callback(putpat_channels);
               } catch(e) {
                Putpat.debug("shit, error!");
                Putpat.debug(e);
                callback(null);
               }
            });
        } else {
            callback(putpat_channels);
        }
    }

    function getClipsForVeequalizer(vq, callback) {
      params = {
          params: {}
      };

      params["params"]["startTime"] = Math.floor(Putpat.getTime().getTime() * 25 / 1000);
      params["params"]["sliders"] = [];
      params["params"]["streamingMethod"] = "http";

      for (var i in  vq["entries"]) {
          veeq = vq["entries"][i];

          if (veeq["slider_value"] != 0) {
              slider = {};
              slider["title"] = veeq["title"];
              slider["type"]  = veeq["type"];
              slider["id"] = veeq["id"];
              slider["color"] = veeq["color"];
              slider["sliderValue"] = veeq["slider_value"];
              slider["decades"] = veeq["decades"];
              params["params"]["sliders"].push(slider);
          }
      }

      Putpat.debug(JSON.stringify(params));
      callAPI(base_url_json + "?method=Veequalizer.clips", "POST", JSON.stringify(params), false, function(result){
          json = JSON.parse(result);
          callback(json);
      });
    }

    return {
        VERSION: VERSION,

        'getPlayerId': function() {
          return playerId;
        },

        'login': function(user, callback) {
           init(function(success) {
               if (success) {
                  doLogin(user, callback);
               }
            });
        },

        'getChannels' : function(callback) {
            doFetchChannels(callback);
        },

        'init' : function(callback) {
          callAPI(base_url_json + "?method=Player.getConfig&client_id="+encodeURIComponent(CLIENT_STRING)+
                  "&app_token=ce_html&app_version="+encodeURIComponent(Putpat.VERSION)+
                  "&partner_id=1&tags="+encodeURIComponent(Device.name.toLowerCase()) + "," +
                  encodeURIComponent(DEVICE_TYPE), "GET", null, true, function(data) {
            var json = JSON.parse(data);
            playerId = json["player_config"]["id"]
            Putpat.debug("Player ID : " + playerId);
            callback();
          }, true, false);
        },

        'initPlayerAdvertisement' : function(file, callback) {
            var url = "http://files.putpat.tv/samsung/advertisement/" + file;

            Putpat.debug("Init player ads with " + file);
            callAPI(url, "GET", null, true, function(data) {
                try {
                  Putpat.playerAdvert = JSON.parse(data);
                } catch (error) {
                  Putpat.debug("Fehler: " + error);
                }
                callback();
            }, false, false);
        },

        'trySignup' : function(user, callback) {
            Putpat.login(user, function(success, message) {
                Tracking.trackAction("user_action", "login");
                if (callback) {
                    callback(success, message);
                }
            });
        },

        'isAssetBanned': function(asset_token, callback) {
            callAPI(base_url_json + "?method=User.asset_ban_status_by_token&asset_token=" + encodeURIComponent(asset_token),
                "GET", null, false, callback);
        },

        'getClipsForChannel': function(channel, callback) {
            callAPI(base_url_json + "?method=Channel.clips&streamingMethod=http&channelId=" + channel["id"], "GET", null, false,callback);
        },

        'getTime' : function() {
            return new Date(new Date().getTime() - Putpat.timeDifference);
        },

        'getVeequalizers' : function(callback) {
            if (Putpat.veequalizers != null) {
                callback(Putpat.veequalizers);
            } else {
                doFetchVeequalizer(callback);
            }
        },

        'getCurrentVeequalizer' : function(callback) {
            Putpat.getVeequalizers(function() {
                for (var i in Putpat.veequalizers) {
                    vq = Putpat.veequalizers[i];
                    if (vq != null && vq["vq_type"] == "user") {
                        callback(vq);
                        return;
                    }
                }
                if (Putpat.veequalizers.length > 0) {
                  callback(Putpat.veequalizers[0]);
                } else {
                  callback(null);
                }
            });
        },

        'getVeequalizedClips' : function(vq, callback) {
            getClipsForVeequalizer(vq, callback);
        },

        'getClipRating' : function(clip, callback) {
            callAPI(base_url_json + "?method=User.clipRating&asset_id=" + encodeURIComponent(clip["asset_id"]), "GET", null, false, function(result) {
                json = JSON.parse(result);
                callback(json["clip_rating"]);
            });
        },

        'setClipRating' : function(clip, rating, callback) {
            callAPI(base_url_json + "?method=User.rateClip&asset_id=" + clip["asset_id"] + "&rating=" + rating, "POST", " ", false, callback);
        },

        'getAssetInfo' : function(asset_id, callback) {
            callAPI(base_url_json + "?method=Asset.get&asset_id=" + asset_id, "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'getTourdates' : function(artist_id, callback) {
            callAPI(base_url_json + "?method=Tourdates.dates&artistId=" + artist_id, "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'trackIvwUi' : function() {
          try {
            if (Putpat.playerAdvert && Putpat.playerAdvert["enable_tracking"] == 1) {
              var random = Math.floor(Math.random()*99999);

              var url = "http://putpattv.ivwbox.de/cgi-bin/ivw/CP/putpat_ui_in;?r=http%3A%2F%2Fwww.putpat.tv&d="+random;

              if (Putpat.playerAdvert["ivw"] && Putpat.playerAdvert["ivw"]["ui"]) {
                url = Putpat.playerAdvert["ivw"]["ui"].replace("{RANDOM}", random);
              }

              callAPIwithFormat(url, "GET", null, true, null, null, false, false);
            }
          } catch(e) {
            reportError(e, "unkown", "Putpat#trackIvwUi", "");
          }
        },

        'trackIvwVideo' : function() {
          try {
            if (Putpat.playerAdvert && Putpat.playerAdvert["enable_tracking"] == 1) {
              var random = Math.floor(Math.random()*99999);
              var url = "http://putpattv.ivwbox.de/cgi-bin/ivw/CP/putpat_video;?r=http%3A%2F%2Fwww.putpat.tv&d="+random;

              if (Putpat.playerAdvert["ivw"] && Putpat.playerAdvert["ivw"]["video"]) {
                url = Putpat.playerAdvert["ivw"]["video"].replace("{RANDOM}", random);
              }

              callAPIwithFormat(url, "GET", null, true, null, null, false, false);
            }
          } catch(e) {
            reportError(e, "unkown", "Putpat#trackIvwVideo", "");
          }
        },

        'reportVideo': function(channelId, assetId, seconds, callback) {
          try {
            report = {
              params: {}
            };

            report["params"]["finished"] = false;
            report["params"]["secondsPlayed"] = Math.round(seconds);
            report["params"]["assetId"] = assetId;
            report["params"]["channelId"] = channelId;

            callAPI(base_url_json + "?method=User.addVideoHistoryItem", "POST", JSON.stringify(report), false, callback);
          } catch(e) {
            reportError(e, "unkown", "Putpat#reportVideo", "");
          }
        },

        'getCompleter' : function(text, callback) {
            callAPI(base_url_json + "?method=Veequalizer.completer&searchterm=" + encodeURIComponent(text), "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'getClipRatings' : function(callback) {
            callAPI(base_url_json + "?method=User.clipRatings", "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'getNewsDetails' : function(id, callback) {
            callAPI(base_url_json + "?method=News.fetchNews&id="+id, "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'getNews' : function(offset, callback) {
            callAPI(base_url_json + "?method=News.articles&offset="+offset+"&subdomain=putpat&count=5", "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'getNewVideoReleases' : function(callback) {
            callAPI(base_url_json + "?method=News.videoReleases&offset=0&count=6&subdomain=putpat", "GET", null, false, function(result) {
                callback(JSON.parse(result));
            });
        },

        'updateVeequalizer': function(veq, callback) {
          try {
            if (veq["vq_type"] == "user") {
              vq = {
                params: {}
              };

              vq["params"]["entries"] = veq["entries"];
              vq["params"]["name"] = veq["name"];
              vq["params"]["veequalizerPresetId"] = veq["id"];

              callAPI(base_url_json + "?method=User.updateVeequalizerPreset", "POST", JSON.stringify(vq), false, function(){
                Putpat.getCurrentVeequalizer(function(current){
                  var index = Putpat.veequalizers.indexOf(current);
                  Putpat.veequalizers.splice(index, 1);
                  Putpat.veequalizers.reverse();
                  Putpat.veequalizers.push(veq);
                  Putpat.veequalizers.reverse();
                });
                Channels.resetCurrentVq();
                callback();
              });
            }
          } catch(e) {
            reportError(e, "unkown", "Putpat#updateVeequalizer", "");
          }
        },

        'debug': function(msg) {
            if (Device.name == "Samsung") {
              alert(msg);
            }

            /*
            var dbgUrl = "http://10.0.220.145:9292/?debug=" + encodeURIComponent(msg);
            XHRPool.getXHR(function(xhr) {
              xhr.open("GET", dbgUrl);
              xhr.setRequestHeader("Content-Type", "text/xml");
              xhr.onreadystatechange = function() {
                if (xhr.readyState == 4) {
                  XHRPool.done(xhr);
                }
              };
              xhr.send();
            });
            */

            if (typeof console == "object") {
              console.log(msg);
            }

            var el = document.getElementById('status');
            if (el != null && msg != null) {
              if (Putpat.debugMessage == null) {
                Putpat.debugMessage = [];
              }

              if (msg.length > 40) {
                msg = msg.substring(0, 40);
              }

              Putpat.debugMessage.push(msg);

              var start = Putpat.debugMessage.length - 30;
              var end = Putpat.debugMessage.length;

              if (start < 0) {
                start = 0;
              }

              var debugText = Putpat.debugMessage.slice(start, end).reverse().join("<br/>");

              el.innerHTML = debugText;

            }
        },

       'sendAdvertismentLink' : function(url, callback) {
            callAPI(base_url_json + "?method=Marketing.sendDetails&url=" + encodeURIComponent(url), "GET", null, false, function() {
                callback();
            });
        },

        'callUrl' : function(url, openErrorDialog, addClient, callback) {
          if (url != null && url.trim().length > 0) {
            callAPI(url, "GET", null, true, callback, openErrorDialog, addClient);
          } else {
            if (callback) {
              callback();
            }
          }
        },

        'callUrls' : function(urls, openErrorDialog, addClient, callback) {
          if (urls != null && urls != undefined) {
            var url = null;

            while ((url == null || url == undefined || url.trim().length == 0) && urls.length > 0) {
              url = urls.pop();
            }

            if (url != null && url.trim().length > 0) {
              callAPI(url, "GET", null, true, function() {
                if (urls.length > 0) {
                  Putpat.callUrls(urls, openErrorDialog, addClient, callback);
                } else if (callback) {
                  callback();
                }
              }, openErrorDialog, addClient);
            } else if (callback) {
              callback();
            }

          } else if (callback) {
            callback();
          }
        },

        'reportError' : function(errorMessage, device, url, line) {
          // Report als Bild abschicken
          // ...und dabei die letzten 20 im Speicher halten, damit
          // der Garbage Collector vom Maple Browser da nicht zu
          // früh dazwischenfunkt
          var userAgent = "";
          if (navigator && navigator.userAgent) {
            userAgent = navigator.userAgent;
          }

          var imgSrc = "http://rp.putpat.tv/tv_bugs?ERROR=" + encodeURIComponent(errorMessage);
          imgSrc += "&DEVICE=" + encodeURIComponent(device);
          imgSrc += "&URL=" + encodeURIComponent(url);
          imgSrc += "&LINE=" + encodeURIComponent(line);
          if (userAgent) {
            imgSrc += "&AGENT=" + encodeURIComponent(userAgent);
          }

          var img = new Image();
          img.src = imgSrc;

          if (reportErrorImages.push(img) > 20) {
            reportErrorImages.shift();
          }
        },

        'callApi' : function(method, params, callback, anonymous) {
           var paramString = "";

           $.each(Utils.hashKeys(params), function(i, key) {
             paramString+= "&" + key + "="+encodeURIComponent(params[key]);
           });

           var addClient = true;
           if (!Utils.isStringEmpty(params["client"])) {
             addClient = false;
           }

           if (anonymous == null || anonymous == undefined) {
             anonymous = false;
           }

           callAPI(base_url_json + "?method="+method+paramString, "GET", null, anonymous, function(result) {
               callback(JSON.parse(result));
           }, null, addClient);
        },

        'getCurrentQuality' : function () {
            var quality = Settings.getValue("video_quality");
            if (quality == null || (["medium", "low"].indexOf(quality) == -1)) {
              quality = "medium";
            }

            return quality;
        },

        'setCurrentQuality' : function(quality) {
            if (quality == null || (["medium", "low"].indexOf(quality) == -1)) {
              quality = "medium";
            }

            Settings.setValue("video_quality", quality);
        },

        'userCornerLogo' : function() {
          return userCornerLogo;
        },

        'user': function() {
          return user;
        },

        'reportAction': function(type, value) {
          Tracking.trackPutpat(type, value);
        }
    };
}();
