String.prototype.trim = function () {
    return this.replace(/^(\s|\n|\t)*/, "").replace(/(\s|\n|\t)*$/, "");
}


var Utils = {

  'hashKeys' : function(hash) {
    var keys = [];
    for(var i in hash) if (hash.hasOwnProperty(i)) {
      keys.push(i);
    }
    return keys;
  },

  'fillStringLeft' : function(str, length, filler) {
    while (str.length < length) {
      str = filler + str;
    }
    return str;
  },

	'getPathFor': function(id, prefix) {
    str = ""+id;
    str = Utils.fillStringLeft(str, 7, 0);
    return str.substring(0,5) + "/" + str + "/" + prefix + str;
	},

  'contains': function(array, obj) {
      var i = array.length;
      while (i--) {
          if (array[i] === obj) {
              return true;
          }
      }
      return false;
  },
    
  'removeStyle' : function(element, style) {
      if (element != null && element.getAttribute("class") != null) {
          classes = element.getAttribute("class").split(" ");
          newclass = "";

          for (var i in classes) {
            if (classes[i] != style) {
                newclass += classes[i] + " ";
            }
          }

          element.setAttribute("class", newclass);
      }
  },
  
  'addStyle' : function(element, style) {
      if (element != null) {
          news = element.getAttribute("class");
          classes = [];
          if (news != null) {
              classes = news.split(" ");
          } else {
              news = "";
          }

          if (!Utils.contains(classes, style)) {
              element.setAttribute("class", news + " " + style);
          }
      }
  },
  
  
  'fixSelection' : function(value, min, max, wrapAround) {
      if (value < min) {
          if (wrapAround) {
              value = max;
          } else {
              value = min;
          }
      }

      if (value > max) {
          if (wrapAround) {
              value = min;
          } else {
              value = max;
          }
      }

      return value;
  },
  
  'createUUID' : function() {
      // http://www.ietf.org/rfc/rfc4122.txt
      var s = [];
      var hexDigits = "0123456789ABCDEF";
      for (var i = 0; i < 32; i++) {
        s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
      }
      s[12] = "4";  // bits 12-15 of the time_hi_and_version field to 0010
      s[16] = hexDigits.substr((s[16] & 0x3) | 0x8, 1);  // bits 6-7 of the clock_seq_hi_and_reserved to 01

      var uuid = s.join("");
      return uuid;
  },

  'randomString' : function(length) {
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for( var i=0; i < length; i++ )
        text += possible.charAt(Math.floor(Math.random() * possible.length));

    return text;
  },

  'shuffleArray' : function(o){ //v1.0
      for(var j, x, i = o.length; i; j = parseInt(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
      return o;
  },

  'isStringEmpty' : function(string) {
    if ((string == null) || (string == undefined) || (string.length == 0)) {
      return true;
    }
    return false;
  },

  'convertColor': function(color) {
    if (color != null) {
      color = color.replace(/^0x/,"#");
      return color;
    }
    return "#000000"
  },

  'clone' : function(o) {
     var newObj = (o instanceof Array) ? [] : {};
     for (i in o) {
       if (i == 'clone') continue;
       if (o[i] && typeof o[i] == "object") {
         newObj[i] = Utils.clone(o[i]);
       } else {
         newObj[i] = o[i]
       }
     }
     return newObj;
  },

  'replaceWithHTMLEntities' : function(str) {
    if (str == null) {
      return null;
    }
    return str.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;");
  },

  'paramFromUrl': function(name) {
    name = name.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
    var regexS = "[\\?&]"+name+"=([^&#]*)";
    var regex = new RegExp( regexS );
    var results = regex.exec( window.location.href );
    if (results == null) {
      return "";
    };
    return results[1];
  },

  'base64': function(input) {
    var keyStr = "ABCDEFGHIJKLMNOP" +
      "QRSTUVWXYZabcdef" +
      "ghijklmnopqrstuv" +
      "wxyz0123456789+/" +
      "=";

    input = escape(input);
    var output = "";
    var chr1, chr2, chr3 = "";
    var enc1, enc2, enc3, enc4 = "";
    var i = 0;

    do {
      chr1 = input.charCodeAt(i++);
      chr2 = input.charCodeAt(i++);
      chr3 = input.charCodeAt(i++);

      enc1 = chr1 >> 2;
      enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
      enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
      enc4 = chr3 & 63;

      if (isNaN(chr2)) {
        enc3 = enc4 = 64;
      } else if (isNaN(chr3)) {
        enc4 = 64;
      }

      output = output +
        keyStr.charAt(enc1) +
        keyStr.charAt(enc2) +
        keyStr.charAt(enc3) +
        keyStr.charAt(enc4);
      chr1 = chr2 = chr3 = "";
      enc1 = enc2 = enc3 = enc4 = "";
    } while (i < input.length);

    return output;
  }

};
