/**
 * Auf Samsungs werden die Einstellungen in einer
 * Datei gespeichert
 */
Settings = function () {
  var options = null;
  var current_options_version = 1;


  function getWidgetId() {
    if (curWidget.id != null) {
      return curWidget.id;
    } else {
      return "putpat"
    }
  }


  function fetchConfig() {
    try {
      if (options == null) {
        filesystem = new FileSystem();

        if (!filesystem.isValidCommonPath(getWidgetId())) {
            filesystem.createCommonDir(getWidgetId());
        }

        // Alte Settings wurden nicht im Unterverzeichnis gespeichert,
        // dafür gibt es diesen Migrationscode :)
        var oldSystem = false;
        var json = null;
        try {
          file = filesystem.openCommonFile("putpat_settings.json", "r");
          json = file.readAll();
          filesystem.closeCommonFile(file);
          filesystem.deleteCommonFile("putpat_settings.json");
          oldSystem = true;
        } catch (e) {
        }

        if (json == null) {
          file = filesystem.openCommonFile(getWidgetId() + "/putpat_settings.json", "r");
          json = file.readAll();
          filesystem.closeCommonFile(file);
        }

        if (json != null) {
          options = JSON.parse(json);
        } else {
          initConfig();
        }

        if (options["users"] == null) {
          options["users"] = [];
        }

        if (oldSystem) {
          saveConfig();
        }
      }
    } catch (e) {
      alert("Fehler!!------------------------------Fetch Config--------------<");
      alert(e);
      initConfig();
    }

  }

  function initConfig() {
    options = {
      'version': current_options_version,
    }

    saveConfig();
  };

  function saveConfig() {
    try {
      filesystem = new FileSystem();
      json = JSON.stringify(options);
      file = filesystem.openCommonFile(getWidgetId() + "/putpat_settings.json", "w");
      file.writeAll(json);
      filesystem.closeCommonFile(file);
    } catch (e) {
      alert("Fehler!!------------------------------------Save Config--------<");
      alert(e);
    }
  }

  function userIndex(user, array) {
    for (var i in array) {
      if (user["type"] == "UUID" && user["uuid"] == array[i]["uuid"]) {
        return i;
      } else if (user["type"] == "LOGIN" && user["login"] == array[i]["login"]) {
        return i;
      }
    }
    return -1;
  }

  return {
    'getValue': function (key) {
      fetchConfig();
      return options[key];
    },

    'setValue': function (key, value) {
      fetchConfig();
      if (options[key] != value) {
        options[key] = value;
        saveConfig();
      }
    },

    'getCurrentUser': function () {
      fetchConfig();
      return options["currentUser"];
    },

    'setCurrentUser': function (user) {
      fetchConfig();
      options["currentUser"] = user;

      if (userIndex(user, options["users"]) == -1) {
        Settings.addUser(user);
      } else {
        saveConfig();
      }
    },

    'getUsers': function () {
      fetchConfig();
      return options["users"];
    },

    'addUser': function (user) {
      fetchConfig();
      if (options["users"] == null) {
        options["users"] = [];
      }
      options["users"].push(user);
      saveConfig();
    },

    'removeUser': function (user) {
      fetchConfig();
      var pos = userIndex(user, options["users"]);
      if (pos != -1) {
        options["users"].splice(pos, 1);
        saveConfig();
      }
    },

    'updateUser': function (user) {
      Settings.removeUser(user);
      Settings.addUser(user);
    },

    'save': function () {
      saveConfig();
    }

  }
}();
