/**
 * Das hier ist die Kern-Klasse des Views.
 * Hier wird bestimmt, welcher Clip gerade gezeigt
 * wird usw
 */
var ChannelView = {
    currentChannel: null,
    lastClip: null,
    currentClip: null,
    currentAdvert : null,
    currentView: null,

    liveView: false,

    // Video Position und Größe
    videoX : 0,
    videoY : 0,
    videoHeight : 540,
    videoWidth : 960,

    // Welcher Modus? VIDEOCLIP oder ADVERT?
    mode : "VIDEOCLIP",
    // Wieviele Videos wurden schon abgespielt
    videoCount : -1,
    // Wurde der jetzt gerade laufende Clip schon gezählt?
    // Gezählt wird ab 15. Sekunden abspieldauer
    clipCounted : false,
    isPaused: false,
    adsAllowed: true,
    // Lädt gerade ein neuer Clip?
    updatingClip: false,
    adClient: null,

    // Initalisiert den ChannelView
    'show': function() {
        Putpat.trackIvwUi()
        $("#putpat_view").append("<div id='channellogo'></div>");
        ChannelView.adClient = Adclient(function() {
          Putpat.getChannels(function(channels) {
              ChannelView.currentChannel = null;

              try {
                // Wenn eine Channel ID per URL Parameter mitgegeben wird,
                // diesen benutzen
                var channelId = Utils.paramFromUrl("channel_id");

                for (i in channels) {
                  if (channels.hasOwnProperty(i)) {
                    if (channels[i].id == channelId) {
                      ChannelView.currentChannel = channels[i];
                    }
                  }
                }
              } catch(e) {
                Putpat.reportError(e, "unknown", "channel_view.js", 54);
              }

              try {
                // Ansonsten mit VQ Channel starten
                if (ChannelView.currentChannel == null) {
                  ChannelView.currentChannel = channels[0];
                }

                Player.stopCallback = function() {
                  ChannelView.updateClip();
                };
                Player.timeCallback  = ChannelView.updateTimeIndex;
                PromotionOverlay.startCounter();
              } catch(e) {
                Putpat.reportError(e, "unknown", "channel_view.js", 69);
              }

              try {
                // Wenn ein AssetToken übergeben wurde in der URL,
                // diesen benutzen
                var asset_token = Utils.paramFromUrl('titleid');

                if (asset_token != "" && asset_token != null) {
                  ChannelView.startAssetToken(asset_token);
                } else {

                  /*
                   * Wenn KEIN AssetToken übergeben wurde und wir NICHT explizit auf
                   * einen gewissen Channel gesprungen sind => nach Startclip schauen
                   */
                  if (ChannelView.currentChannel.id === 0 &&
                      Putpat.playerAdvert &&
                      Putpat.playerAdvert["first_video_on_app_start"] &&
                      Putpat.playerAdvert["first_video_on_app_start"]["asset_token"] &&
                      Putpat.playerAdvert["first_video_on_app_start"]["asset_token"] !== "") {

                        var firstVideoToken = Putpat.playerAdvert["first_video_on_app_start"]["asset_token"];

                        // User-Rating für das Asset einholen:
                        Putpat.isAssetBanned(firstVideoToken, function(data) {
                          var isAssetBanned = true;

                          try {
                            data = JSON.parse(data);
                            isAssetBanned = !!(data["asset_ban_status"]["banned"]);
                          } catch(e) {
                            Putpat.debug(e);
                            // hier ist was schiefgelaufen, nicht zeigen!
                            isAssetBanned = true;
                          }

                          Putpat.debug("asset gebannt? " + isAssetBanned);

                          // Nur zeigen wenn der User den Clip & Artist nicht gebannt hat
                          if (!isAssetBanned) {
                            ChannelView.startAssetToken(firstVideoToken);

                            if (Putpat.playerAdvert && Putpat.playerAdvert["first_video_on_app_start"]["tracking"]) {
                              var trackings = Putpat.playerAdvert["first_video_on_app_start"]["tracking"];

                              for (trackEntry in trackings) {
                                Tracking.trackUrl(trackings[trackEntry]);
                              }
                            }
                          } else {
                            // Es ist zwar ein Startclip da, aber der User hat dieses Asset gebannt
                            // => nicht zeigen!
                            ChannelView.updateClip();
                          }
                        });

                  } else {
                    Putpat.debug("Habe kein Token gefunden!");
                    // Keine besondere Einstellung, mit aktuellem Channel starten
                    ChannelView.updateClip();
                  }

                }
              } catch(e) {
                Putpat.debug(e);
                Putpat.reportError(e, "unknown", "channel_view.js", 82);
              }
          });
        });

        ChannelOverlay.show();
    },

    'resetCurrentClip' : function() {
      try {
        if (ChannelView.mode == "ADVERT") {
          ChannelView.mode = "VIDEOCLIP";
          AdvertisementPanel.hide();
        }
        ChannelView.lastClip = ChannelView.currentClip;
        ChannelView.currentClip = null;
        ChannelView.currentAdvert = null;
      } catch(e) {
        Putpat.reportError(e, "unknown", "channel_view.js", 98);
      }
    },

    'submitCurrentClip' : function(callback) {
        if (ChannelView.currentClip != null && ChannelView.currentAdvert == null) {
            var clip        = ChannelView.currentClip;
            var channel     = ChannelView.currentChannel;
            var lastOffset  = ChannelView.lastOffset;
            var startOffset = ChannelView.startOffset;

            if (clip["live_streaming"]) {
              // Do nothing, it is streaming :)
              callback();
            } else if (startOffset > 0 && (lastOffset - startOffset) > 1) {
                Putpat.reportVideo(channel["id"], clip["asset_id"], (lastOffset - startOffset) + 1, function () {
                  ChannelView.resetCurrentClip();
                  if (callback != null) {
                      callback();
                  }
                });
            } else {
              ChannelView.resetCurrentClip();
              if (callback != null) {
                 callback();
               }
            }
        } else {
          ChannelView.resetCurrentClip();
          if (callback != null) {
            callback();
          }
        }
    },

    'updateTimeIndex' : function (time) {
        if (Player.state == Player.STOPPED) {
          return;
        }

        if (Putpat.removeSplashTimer == null || Putpat.removeSplashTimer < new Date()) {
          Splash.hide();
        }

        if (Loader.isRunning()) {
          Loader.stop();
        }

        var seconds = time / 1000;

        if (seconds >= 15 && !ChannelView.clipCounted && ChannelView.mode != "ADVERT") {
          Putpat.debug("*********** VIDEO COUNT ************ " + seconds + " ---- ");
          ChannelView.videoCount++;
          ChannelView.clipCounted = true;
        }

        if (ChannelView.startOffset == -1) {
            ChannelView.startOffset = new Date().getTime() / 1000;
        }

        var secondsDiff = 0;
        if (ChannelView.lastOffset > 0) {
          secondsDiff = (new Date().getTime() / 1000) - ChannelView.lastOffset;
        }

        if (secondsDiff > 0 && Player.state != Player.PAUSED) {
          ChannelView.isPaused = false;
        }
        ChannelView.lastOffset = new Date().getTime() / 1000;

        if (ChannelView.currentClip != null && ChannelView.currentAdvert == null) {
           var real_duration = ChannelView.currentClip["start_offset"] + ChannelView.currentClip["duration"];
           MessageQueue.sendMessage("play_position", {pos: seconds, duration: real_duration});
           if (seconds >= real_duration) {
               ChannelView.updateClip();
           }
        } else if (ChannelView.currentAdvert != null){
           MessageQueue.sendMessage("advert_position", {time: time, duration: Player.getDuration()});
        }

    },

    'playAdvert': function(status, ad) {
      try {
        ChannelView.mode = "ADVERT";
        Putpat.debug("try to play advert");
        Putpat.debug("---------------------- ADVERT");
        Putpat.debug(status);

        if (status == "OK" && ad.video_url != null && ad.video_url.length > 0) {
          try {
            AdvertisementPanel.show(ad);
          } catch(e) {
            Putpat.reportError(e, "unknown", "channel_view.js", 190);
          }
        } else {
          try {
            Putpat.debug("no ad found :(");
            ChannelView.resetCurrentClip();
            ChannelView.updateClip();
          } catch(e) {
            Putpat.reportError(e, "unknown", "channel_view.js", 198);
          }
        }
      } catch(e) {
        Putpat.reportError(e, "unknown", "channel_view.js", 203);
      }
    },

    'updateChannelLogo' : function() {
        if (ChannelView.currentChannel.id == 0) {
          if ($("#deinChannel").length == 0) {
            $("#channellogo").empty();
            $("#channellogo").append("<img id='deinChannel' src='"+Putpat.userCornerLogo()+"'/>");
          }
        } else {
          var src = $("#channelGraphic").attr("src");
          var target = "http://files.putpat.tv/artwork/channelgraphics/"+ChannelView.currentChannel.id+"/channellogo_100.png";
          if (src != target) {
            $("#channellogo").empty();
            $("#channellogo").append("<img id='channelGraphic' src='"+target+"'/>");
          }
        }
    },

    'updateChannelShowPicture' : function() {
      var removedEl = $("#channelShowPicture").remove();
      if (ChannelView.currentClip.clipInfo.clip_container != null &&
            ChannelView.currentClip.clipInfo.clip_container.display_values == "PICTURE" &&
            !Utils.isStringEmpty(ChannelView.currentClip.clipInfo.clip_container.picture)) {

        var url = ChannelView.currentClip.clipInfo.clip_container.picture;

        if (url.substring(0, 7) != "http://") {
          url = "http://files.putpat.tv/artwork/playlist_pictures/"+url+"_small.png";
        }

        $("#channellogo").append("<div id='channelShowPicture'>"+
                            "<img src='"+ url + "'/>" +
                          "</div>"
                        );
      }
    },

    'updateClip' : function() {
      try {
        if (ChannelView.updatingClip) {
          return;
        }
        ChannelView.updatingClip = true;
        Player.stopVideo(false);

        if (ChannelView.liveView) {
          LiveEventChannel.updateClip();
          return;
        }

        if (ChannelView.mode == "ERROR") {
          return;
        }


        Loader.start();
        ChannelView.submitCurrentClip(function() {
          ChannelView.isPaused = false;
          Putpat.debug("Loading Clips... Mode : "+ ChannelView.mode);
          ChannelView.updateChannelLogo();

          if (ChannelView.videoCount == -1 && ChannelView.currentChannel && Channels.isCurrentChannelPlayingAds()) {
            Putpat.debug("getAdvertisement#1");

            ChannelView.mode = "ADVERT";
            ChannelView.videoCount = 0;
            ChannelView.adClient.getAdvertisement("onstart", function(status, ad) {
              try {
                ChannelView.updatingClip = false;
                ChannelView.playAdvert(status, ad);
              } catch(e) {
                Putpat.reportError(e, "unknown", "channel_view.js", 272);
              }
            });
          } else if (ChannelView.adClient.shouldPlayAdvert(ChannelView.videoCount, AdvertisementPanel.adsPlayed()) &&
            (ChannelView.mode == "VIDEOCLIP") && ChannelView.adsAllowed && Channels.isCurrentChannelPlayingAds()) {
              Putpat.debug("getAdvertisement#2");

              ChannelView.mode = "ADVERT";
              ChannelView.videoCount = 0;
              ChannelView.adClient.getAdvertisement("preroll", function(status, ad) {
                try {
                  Putpat.debug("----------------- start playing");
                  ChannelView.updatingClip = false;
                  ChannelView.playAdvert(status, ad);
                } catch(e) {
                  Putpat.reportError(e, "unknown", "channel_view.js", 285);
                }
              });
            } else {
              Putpat.debug("getCurrentClipForChannel");

              Channels.getCurrentClipForChannel(ChannelView.currentChannel, function(clip) {
                Putpat.debug("getCurrentClipForChannelInner");

                try {
                  if (clip == null) {
                    return;
                  }
                  if (clip["artist"] == null) {
                    clip["artist"] = "";
                  }
                  if (clip["title"] == null) {
                    clip["title"] = "";
                  }

                  ChannelView.startOffset = -1;
                  ChannelView.lastOffset = new Date().getTime() / 1000;
                  ChannelView.currentClip = clip;
                  ChannelView.clipCounted = false;
                  ChannelView.updatingClip = false;
                  Putpat.trackIvwVideo();

                  Tracking.trackStat("video_play", "asset_id="+ clip["asset_id"] + "&video_file_id=" + clip["video_file_id"]);

                  MessageQueue.sendMessage("new_clip", {artist: clip["artist"], title: clip["title"]});

                  ChannelView.updateChannelShowPicture();
                } catch(e) {
                  Putpat.debug(e);
                  Putpat.reportError(e, "unknown", "channel_view.js", 314);
                }

                Putpat.debug("Clip Counter : " + ChannelView.videoCount + " --- " + "   " + ChannelView.mode);
                try {
                  Player.setVideoURL(clip["url_" + Putpat.getCurrentQuality()]);
                  ChannelView.updatePlayerSize();
                  Player.playVideo(0);
                } catch (e) {
                  Putpat.debug(e.message);
                  Putpat.reportError(e, "unknown", "channel_view.js", 324);
                }
              });
            }
        });
      } catch(e) {
        Putpat.reportError(e, "unkown", "channel_view#updateClip", "");
      }
    },

    'calculateVideoSize' : function() {
      try {
        if (ChannelView.minimized) {
          ChannelView.videoX = Positions.player_min[0];
          ChannelView.videoY = Positions.player_min[1];
          ChannelView.videoWidth = Positions.player_min[2];
          ChannelView.videoHeight = Positions.player_min[3];

          var ar = ChannelView.currentClip["aspect_ratio"];
          var origWidth = ChannelView.videoWidth;
          var origHeight = ChannelView.videoHeight;

          if (Positions.player_min[2] < ChannelView.videoHeight * ar) {
            ChannelView.videoWidth = Math.floor(ChannelView.videoHeight * ar);
          } else {
            ChannelView.videoHeight = Math.floor(ChannelView.videoWidth / ar);
          }
          ChannelView.videoY -= ChannelView.videoHeight - Positions.player_min[3];
        } else {
          ChannelView.videoX = Positions.player_max[0];
          ChannelView.videoY = Positions.player_max[1];
          ChannelView.videoWidth = Positions.player_max[2];
          ChannelView.videoHeight = Positions.player_max[3];

          if (Device.scaleVideoRelative) {
            var ar = ChannelView.currentClip["aspect_ratio"];

            if (ChannelView.videoWidth > ChannelView.videoHeight * ar) {
              ChannelView.videoWidth = Math.floor(ChannelView.videoHeight * ar);
            } else {
              ChannelView.videoHeight = Math.floor(ChannelView.videoWidth / ar);
            }

            ChannelView.videoX += (Positions.player_max[2] - ChannelView.videoWidth) / 2;
            ChannelView.videoY += (Positions.player_max[3] - ChannelView.videoHeight) / 2;
          }
        }
      } catch(e) {
        Putpat.reportError(e, "unkown", "channel_view#calculateVideoSize", "");
      }
    },

    'updatePlayerSize': function(clip) {
      try {
        if (ChannelView.currentClip != null) {
          ChannelView.calculateVideoSize();
          Player.setViewportBounds( Math.floor(ChannelView.videoX),
              Math.floor(ChannelView.videoY),
              Math.floor(ChannelView.videoWidth), Math.floor(ChannelView.videoHeight) );
        }
      } catch(e) {
        Putpat.reportError(e, "unkown", "channel_view#updatePlayerSize", "");
      }
    },

    'moveChannel': function(dir) {
      try {
        if (ChannelView.currentChannel != null) {
          Putpat.getChannels(function(channels) {
            selectedChannel = 0;
            for (var i=0;i<channels.length;i++) {
              channel = channels[i];
              if (channel["id"] == ChannelView.currentChannel["id"]) {
                selectedChannel = i;
              }
            }

            selectedChannel += dir;
            if (selectedChannel < 0) {
              selectedChannel = channels.length - 1;
            } else if (selectedChannel >= channels.length) {
              selectedChannel = 0;
            }
            ChannelView.switchToChannel(channels[selectedChannel]);
          });
        }
      } catch(e) {
        Putpat.reportError(e, "unkown", "channel_view#moveChannel", "");
      }
    },

    'shutdownApp' : function(exit, event) {
      ChannelView.submitCurrentClip(function(){
        if (Device.hasExit) {
            Player.stopVideo(false);
            Player.deinit();
            if (exit) {
              widgetAPI.sendExitEvent();
            } else {
              widgetAPI.sendReturnEvent();
            }
            if (event != null) {
              event.preventDefault();
            }
        }
      });
    },

   'switchToChannel' : function(channel, force, resetVq) {
     try {
       if (ChannelView.mode == "ADVERT") {
         return;
       }

       if (force || ChannelView.currentChannel == null || ChannelView.currentChannel.id != channel.id) {
         Tracking.trackAction("user_action", "channel_change", channel.id);
         LiveEventChannel.stop();
         Putpat.debug("stopping and switching to channel");
         Player.stopVideo(false);
         Putpat.debug("---");
         ChannelView.currentChannel = channel;
         if ((channel.id == 0) && (resetVq != false)) {
           Channels.resetCurrentVq();
         }
         if (channel["live_event_id"] != null && channel["live_event_id"] > 0) {
           ChannelView.liveView = true;
           Putpat.debug("*********** LIVE EVENT ID ! ********************");
           LiveEventChannel.start(channel["live_event_id"]);
         } else {
           ChannelView.liveView = false;
           ChannelView.updateClip();
         }
       }
     } catch(e) {
       Putpat.reportError(e, "unkown", "channel_view#switchToChannel", "");
     }
   },

   'startPlaylist' : function(playlistTag, assetId) {
     try {
       if (ChannelView.mode == "ADVERT") {
         return;
       }

       Putpat.debug("loading playlist");
       Channels.loadPlaylist(playlistTag, assetId, function() {
         Putpat.debug("done, now switch to channel");
         Putpat.getChannels(function(channels) {
           Putpat.debug("switchgin");
           ChannelView.switchToChannel(channels[0], true, false);
           Putpat.debug("done switchign");
         });
       });
     } catch(e) {
       Putpat.reportError(e, "unkown", "channel_view#startPlaylist", "");
     }
   },

   'startClip' : function(assetId, token) {
     try {
       if (ChannelView.mode == "ADVERT") {
         return;
       }

       Channels.loadClip(assetId, token, function() {
         Putpat.getChannels(function(channels) {
           ChannelView.switchToChannel(channels[0], true, false);
         });
       });
     } catch(e) {
       Putpat.reportError(e, "unkown", "channel_view#startClip", "");
     }
   },

   'startAssetToken' : function(token) {
     try {
       if (ChannelView.mode == "ADVERT") {
         return;
       }

       Channels.loadAssetToken(token, function() {
         Putpat.getChannels(function(channels) {
           ChannelView.switchToChannel(channels[0], true, false);
         });
       });
     } catch(e) {
       Putpat.reportError(e, "unkown", "channel_view#startAssetToken", "");
     }
   },

   'pauseVideo' : function() {
     try {
       if (!ChannelView.isPaused) {
         Player.pauseVideo();
         ChannelView.isPaused = true;
         MessageQueue.sendMessage("state_pause", true);
       }
     } catch(e) {
       Putpat.reportError(e, "unkown", "channel_view#pauseVideo", "");
     }
   },

   'playVideo' : function() {
     try {
       if (ChannelView.isPaused) {
         Player.resumeVideo();
         ChannelView.isPaused = false;
         MessageQueue.sendMessage("state_play", true);
       }
     } catch(e) {
       Putpat.reportError(e, "unkown", "channel_view#playVideo", "");
     }
   },

   'togglePlayPause' : function() {
     if (ChannelView.isPaused) {
       ChannelView.playVideo();
       Tracking.trackAction("user_action", "play");
     } else {
       ChannelView.pauseVideo();
       Tracking.trackAction("user_action", "pause");
     }
   },

   'skipVideo' : function() {
      if ((ChannelView.currentChannel["id"] == 0)) {
        Tracking.trackAction("user_action", "skip");

        Player.stopVideo(false);
        Channels.nextVQClip();
        ChannelView.updateClip();
      }
    },

    'hideChannelLogo': function() {
       $("#channellogo").hide();
    },

    'showChannelLogo': function() {
       $("#channellogo").show();
    },

    'disableAds' : function() {
      ChannelView.adsAllowed = false;
    },

    'enableAds' : function() {
      ChannelView.adsAllowed = true;
    }
};
