var ControlBar = function() {
  var self = {};

  var BUTTON_ROTATION = [
      'button_info',
      'button_rate',
      'button_skip',
      'button_vq',
      'button_channels',
      'button_news',
      'button_clips',
      'button_pause',
      'button_settings',
      'button_profile',
      'button_exit'
  ];

  var BUTTON_SCREENS = {
    'button_info' : ArtistInfo,
    'button_rate' : RatingBar,
    'button_channels' : ChannelSelect,
    'button_clips': ClipsPanel,
    'button_news' : NewsPanel,
    'button_settings' : SettingsPanel,
    'button_vq'   : VqPanel,
    'button_profile': ProfilePanel
  };

  var selected = '';
  var activated = '';
  var oldSelected = null;
  var currentScreen = null;
  var inactivityTimer = null;
  var firstTimeOpened = true;
  var hasFocus = false;
  var oldHandlers = null;

  function resetButtonState() {
    $('#controlbar .button').each(function() {
      $(this).attr('src', $(this).attr('src').replace("_hover.png", ".png"));
      $(this).removeClass('selected_button');
    });
    var removedEl = $(".controlbar_tooltip").remove();
  }

  function selectButton(name) {
    if ($("#" + name).hasClass("disabled_button")) {
      moveSelection(1, BUTTON_ROTATION[name]);
      return;
    }

    resetButtonState();

    Putpat.debug(name);

    if (Utils.isStringEmpty(name)) {
      return;
    }

    // Button selektieren
    $("#"+name).attr('src', $("#"+name).attr('src').replace(".png", "_hover.png"));
    $("#"+name).addClass('selected_button');
    selected = name;

    // Tooltip anzeigen, wenn notwendig
    if (activated != selected && Positions.controlbar.tooltips[selected] != null) {
      var imagename = selected.replace("button", "tooltip");
      var tooltipName = selected;
      if (imagename == "tooltip_pause" && ChannelView.isPaused) {
        imagename = "tooltip_play";
        tooltipName = "button_play";
      }

      var pos = Positions.controlbar.tooltips[tooltipName];
      var buttonPos = $("#"+name).offset().left;
      $("#putpat_view").append(
                "<div class='controlbar_tooltip'"+
                  " style='top:"+pos[1]+"px;left:"+(buttonPos + pos[0])+"px;'>"+
                    "<img src='"+Positions.size+"images/controlbar/"+imagename+".png'/>"+
                "</div>"
      );
    }

  }

  function moveSelection(dir, current) {
    if (current == null) {
      current = selected;
    }
    var newitem = BUTTON_ROTATION.indexOf(current);
    do {
      newitem += dir;

      if (newitem < 0) {
        newitem = BUTTON_ROTATION.length - 1;
      } else if (newitem >= BUTTON_ROTATION.length) {
        newitem = 0;
      }

    } while  ($("#"+BUTTON_ROTATION[newitem]).length == 0);

    if ($("#"+BUTTON_ROTATION[newitem]).hasClass("disabled_button")) {
      moveSelection(dir, BUTTON_ROTATION[newitem]);
    } else {
      selectButton(BUTTON_ROTATION[newitem]);
    }
  }

  function keyboardHandler(event) {
     var keycode = event.keyCode;
     resetInactivityTimer();

     if (hasFocus) {
       switch(keycode) {
           case KEYS.KEY_UP:
               if (currentScreen != null) {
                 resetButtonState();
                 currentScreen.focus();
                 event.preventDefault();
               } else if (PromotionOverlay.isVisible()){
                 selectPromotion();
                 event.preventDefault();
               }
               break;
           case KEYS.KEY_LEFT:
               moveSelection(-1);
               event.preventDefault();
               break;
           case KEYS.KEY_RIGHT:
               moveSelection(+1);
               event.preventDefault();
               break;
           case KEYS.KEY_ENTER:
               executeButton();
               event.preventDefault();
               break;
           case KEYS.KEY_DOWN:
           case KEYS.KEY_RETURN:
               if (Device.closeOnReturnIfMouseActive && InputHandler.mouseActive) {
                 ChannelView.shutdownApp(false, event);
               } else {
                 self.hide();
               }
               event.preventDefault();
               break;
           case KEYS.KEY_PANEL_CH_UP:
           case KEYS.KEY_CH_UP:
               ChannelView.moveChannel(1);
               event.preventDefault();
               break;
           case KEYS.KEY_PANEL_CH_DOWN:
           case KEYS.KEY_CH_DOWN:
               ChannelView.moveChannel(-1);
               event.preventDefault();
               break;
        }
     }
  }

  function selectPromotion() {
    oldSelected = selected;
    selectButton(null);
    currentScreen = PromotionOverlay;
    currentScreen.focus();
  }

  function selectPosition(selected) {
    unselect();

    activated = selected;
    selectButton(selected);

    $('#controlbar .button').each(function() {
      if ($(this).attr('id') != selected) {
        $(this).addClass('dimm_button');
      } else {
        $(this).removeClass('dimm_button');
      }
    });

    var buttonPos = $("#"+selected).offset().left;
    var pos = Positions.controlbar.triangle_position[selected] + buttonPos;

    $("#putpat_view").append("<div id='controlbar_drop_shadow'></div>");
    $("#controlbar").append(
          "<div class='select_top_left' style='width:"+pos+"px;'></div>" +
          "<div class='select_top_triangle' style='left:"+pos+"px;'></div>" +
          "<div class='select_top_right' style='left:"+(pos+Positions.controlbar.triangle_size)+"px;width:"+(Positions.screen_size[0] - pos - Positions.controlbar.triangle_size)+"px'></div>"
    );

    $("#controlbar_background").css("top", Positions.controlbar.shadow_gap);
    $("#controlbar_background").css("opacity", "1.0");
  }

  function unselect() {
    activated = null;
    selected = null;
    var removedEl1 = $("#controlbar div.select_top_left").remove();
    var removedEl2 = $("#controlbar div.select_top_triangle").remove();
    var removedEl3 = $("#controlbar div.select_top_right").remove();
    var removedEl4 = $("#controlbar_drop_shadow").remove();

    $("#controlbar_background").css("top", "0px");
    $("#controlbar_background").css("opacity", "0.85");

    if (currentScreen != null) {
      currentScreen.hide();
    }
  }

  function togglePlayPause() {
    ChannelView.togglePlayPause();
  }

  function skipPressed() {
    if (!Loader.isRunning()) {
      ChannelView.skipVideo();
      self.hide();
    }
  }

  function showScreen(screen) {
    hasFocus = false;
    if (["button_info", "button_rate"].indexOf(screen) != -1) {
      if (ChannelView.currentClip == null || ChannelView.currentClip == undefined ||
          ChannelView.currentClip.asset_type != "MusicVideo") {
        return;
      }
    }

    Putpat.trackIvwUi();
    selectPosition(screen);
    TopBar.hide();
    resetButtonState();
    Putpat.debug("button.." + screen);
    currentScreen = BUTTON_SCREENS[screen];
    Putpat.debug(currentScreen == null);
    try {
      currentScreen.show(true);
    } catch (e) {
      Putpat.debug("error 10");
      Putpat.debug(e.message);
    }
  }

  function executeButton() {
    if (selected == "button_pause") {
      togglePlayPause();
    } else if (selected == "button_skip") {
      skipPressed();
    } else if (selected == "button_exit") {
      ChannelView.shutdownApp(true, null);
    } else if (Utils.hashKeys(BUTTON_SCREENS).indexOf(selected) != -1) {
      if (activated != selected) {
        showScreen(selected);
      } else {
        currentScreen.close();
        TopBar.show();
      }
    }
  }

  function updatePlayPauseState() {
    if (ChannelView.isPaused) {
      $("#button_pause").attr("src", Positions.size + "images/controlbar/bt_play.png");
    } else {
      $("#button_pause").attr("src", Positions.size + "images/controlbar/bt_pause.png");
    }
    selectButton(selected);
  }

  function updateSkipState() {
    $("#button_skip").removeClass('disabled_button');
    if (ChannelView.currentChannel.id != 0) {
      $("#button_skip").addClass('disabled_button');
      $("#button_skip").hide();
    } else {
      $("#button_skip").show();
    }
  }

  function updateButtonsForNewClip() {
    if (ChannelView.liveView) {
      if (ChannelView.currentClip.asset_type != "MusicVideo") {
        $("#button_info").addClass('disabled_button');
      }
      $("#button_rate").addClass('disabled_button');
    } else if (ChannelView.currentClip == null || ChannelView.currentClip == undefined ||
        ChannelView.currentClip.asset_type != "MusicVideo") {
      $("#button_info").addClass('disabled_button');
      $("#button_rate").addClass('disabled_button');
    } else {
      $("#button_info").removeClass('disabled_button');
      $("#button_rate").removeClass('disabled_button');
    }

    updateSkipState();
    updatePlayPauseState();
  }

  function messageListener(key, message) {
    if (key == 'play_position') {
      var length = 0;
      if (!isNaN(message["duration"])) {
        var percent = message['pos'] / (message['duration'] / 100);
        length = Math.round(Positions.screen_size[0] / 100 * percent);
      }
      if (length == 0 || !isNaN(length)) {
        $("#controlbar .progressbar").hide();
      } else {
        $("#controlbar .progressbar").show();
        $("#controlbar .progressbar").css("width", length + "px");
      }
    } else if (key == 'new_clip') {
      updateButtonsForNewClip();
      Channels.getCurrentChannelColor(function(color) {
        $("#controlbar .progressbar").css("background-color", color);
      });
    } else if (key == 'state_pause' || key == "state_play") {
      updatePlayPauseState();
    }
  }

  function resetInactivityTimer() {
    if (inactivityTimer != null) {
      clearTimeout(inactivityTimer);
    }
    inactivityTimer = setTimeout(function() {
      if (currentScreen == null) {
        self.hide();
      }
    }, 6000);
  }

  self.show = function() {
    Putpat.trackIvwUi();
    Settings.setValue("newUser", false);
    $("#newUserClick").remove();
    $("#controlbar").remove();
    $(".controlbar_tooltip").remove();
    resetInactivityTimer();
    TopBar.show();
    ChannelView.currentView = "ControlBar";

    MessageQueue.sendMessage("controlbar_visible", {"visible": true});
    MessageQueue.sendMessage("stop_channel_overlay");
    MessageQueue.sendMessage("show_bauchbinde");
    MessageQueue.addListener(messageListener);

    var buttons = "<div class='left_buttons'>"+
                    "<img id='button_info' class='button' src='"+Positions.size + "images/controlbar/bt_info.png'/>" +
                    "<img id='button_rate' class='button' src='"+Positions.size + "images/controlbar/bt_rate.png'/>" +
                    "<img id='button_skip' class='button' src='"+Positions.size + "images/controlbar/bt_skip.png'/>" +
                  "</div>"+
                  "<div class='right_buttons'>" +
                    "<img id='button_pause' class='button' src='"+Positions.size + "images/controlbar/bt_pause.png'/>" +
                    "<img id='button_settings' class='button' src='"+Positions.size + "images/controlbar/bt_settings.png'/>" +
                    "<img id='button_profile' class='button' src='"+Positions.size + "images/controlbar/bt_profile.png'/>";

    if (Device.exitInMenu) {
       buttons += "<img id='button_exit' class='button' src='"+Positions.size + "images/controlbar/bt_exit.png'/>";
    }

    buttons +=     "</div>"+
                    "<div class='middle_buttons'>" +
                      "<img id='button_vq' class='button' src='"+Positions.size + "images/controlbar/bt_vq.png'/>" +
                      "<img id='button_channels' class='button' src='"+Positions.size + "images/controlbar/bt_channels.png'/>";

    if (Device.showNews) {
      buttons +=      "<img id='button_news' class='button' src='"+Positions.size + "images/controlbar/bt_news.png'/>";
    }

    buttons +=        "<img id='button_clips' class='button' src='"+Positions.size + "images/controlbar/bt_clips.png'/>" +
                    "</div>";


    $("#putpat_view").append("<div id='controlbar'>" +
                      "<div id='controlbar_background'></div>"+
                      buttons +
                      "<div class='progressbar'></div>"+
                      "</div>");

    currentScreen = null;
    activated = null;
    updateButtonsForNewClip();

    var startButton = "button_info";

    if (ChannelView.currentClip == null || ChannelView.currentClip == undefined ||
          ChannelView.currentClip.asset_type != "MusicVideo") {

      if (ChannelView.currentChannel.id == 0) {
        startButton = "button_skip";
      } else {
        startButton = "button_vq";
      }

    }

    Channels.getCurrentChannelColor(function(color) {
      $("#controlbar .progressbar").css("background-color", color);
    });

    oldHandlers = InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });

    if (PromotionOverlay.isVisible()){
      selected = startButton;
      selectPromotion();
    } else if (!firstTimeOpened) {
      selectButton(startButton);
    } else {
      firstTimeOpened = false;
      setTimeout(function() {
        selectButton(startButton);
      }, 200);
    }

    hasFocus = true;

    $(".button").mouseover(function() {
      if (Device.hasMouse) {
        resetInactivityTimer();
        hasFocus = true;
        if (currentScreen != null) {
          currentScreen.unfocus();
        }
        selectButton(this.id);
      }
    });
    $(".button").click(function() {
      if (Device.hasMouse) {
        resetInactivityTimer();
        hasFocus = true;
        if (selected != this.id) {
          selectButton(this.id);
        }
        executeButton(this.id);
      }
    });
  }

  self.hide = function() {
    MessageQueue.sendMessage("controlbar_visible", {"visible": false});
    unselect();
    TopBar.hide();
    var removedEl1 = $("#controlbar").remove();
    var removedEl2 = $(".controlbar_tooltip").remove();
    ChannelView.currentView = null;
    InputHandler.replaceKeyHandlers(oldHandlers);
    MessageQueue.removeListener(messageListener);
    MessageQueue.sendMessage("start_channel_overlay");
    if (widgetAPI.hideMouse) {
      widgetAPI.hideMouse();
    }
  }

  self.currentScreenClosed = function() {
    hasFocus = true;
    var abuffer = activated;
    currentScreen = null;
    unselect();
    TopBar.show();
    $('#controlbar .button').each(function() {
      $(this).removeClass('dimm_button');
    });
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });
    if (abuffer == null && oldSelected != null) {
      abuffer = oldSelected;
      oldSelected = null;
    }
    selectButton(abuffer);
    resetInactivityTimer();
  }

  self.focus = function() {
    hasFocus = true;
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });
    selectButton(selected);
  }

  self.unfocus = function() {
    if (hasFocus) {
      hasFocus = false;
      resetButtonState();
    }
  }

  return self;
}();
