var ProfilePanel = function() {
  var self = {};

  var BUTTON_ROTATION = ["button_left", "button_right", "button_close"];

  var BUTTON_SWITCH_ROTATION = ["button_close", "text_user", "text_password", "button_login"];

  var selected = null;
  var mode = "select_boxes";
  var oldTextInput = "";
  var standalone = false;
  var panelTitle = "";
  var panelSubtitle = "";
  var pinCallTimer = null;
  var userField = null;
  var passwordField = null;
  var userFieldInit = false;
  var passwordFieldInit = false;

  function keyboardHandler(event) {
     var keycode = event.keyCode;

     switch(keycode) {
         case KEYS.KEY_LEFT:
             event.preventDefault();
             moveHorizontally(-1);
             break;
         case KEYS.KEY_RIGHT:
             event.preventDefault();
             moveHorizontally(+1);
             break;
         case KEYS.KEY_ENTER:
             event.preventDefault();
             executeButton();
             break;
         case KEYS.KEY_UP:
             event.preventDefault();
             moveVertically(-1);
             break;
         case KEYS.KEY_DOWN:
             event.preventDefault();
             moveVertically(+1);
             break;
         case KEYS.KEY_RETURN:
             event.preventDefault();
             self.close();
             break;
     };
  };

  function executeButton() {
    switch (selected) {
      case "button_close":
        self.close();
        break;
      case "button_left":
        Putpat.trackIvwUi();
        showCompleteProfile();
        break;
      case "button_right":
        Putpat.trackIvwUi();
        showSwitchProfile();
        break;
      case "text_user":
        if (Device.needsOnScreenKeyboard) {
          $(".variant_left_box").hide();
        }
        if (userFieldInit) {
          userField.focus();
        }
        break;
      case "text_password":
        if (Device.needsOnScreenKeyboard) {
          $(".variant_left_box").hide();
        }
        if (passwordFieldInit) {
          passwordField.focus();
        }
        break;
      case "button_login":
        tryLogin();
        break;
    }
  }

  function textInputKeyPressed(textInput, oldText, keyCode) {
      switch(keyCode) {
          case KEYS.KEY_RETURN:
              if (!Device.backKeyIsDelete) {
                 setTimeout(function() {
                    textInput.setText(oldText);
                    $(".variant_left_box").show();
                    textInput.unfocus();
                 }, 200);
                 return true;
              }
              break;
          case KEYS.KEY_UP:
               setTimeout(function() {
                  $(".variant_left_box").show();
                  moveVertically(-1);
               }, 200);
          case KEYS.KEY_ENTER:
               setTimeout(function() {
                 textInput.unfocus();
                 $(".variant_left_box").show();
               }, 200);
               return true;
          case KEYS.KEY_DOWN:
               setTimeout(function() {
                 moveVertically(1);
                 $(".variant_left_box").show();
               }, 200);
               return true;
      }
      return false;
  }

  function userInput(keyCode) {
    textInputKeyPressed(userField, getUsername(), keyCode);
  }

  function passwordInput(keyCode) {
    textInputKeyPressed(passwordField, "", keyCode);
  }

  function loginUser(user) {
     Putpat.trySignup(user, function(success) {
         $(".statusmeldung").empty();
         if (success) {
             Settings.setCurrentUser(user);
             Putpat.getChannels(function(channels) {
               if (!standalone) {
                 ChannelView.switchToChannel(channels[0], true, true);
                 ChannelView.updateChannelLogo();
                 self.close();
                 ControlBar.hide();
               } else {
                 var removedEl = $("#profile_panel").remove();
                 InputHandler.replaceKeyHandlers({
                    keyboardHandler: null
                 });

                 ChannelView.show();
                 ChannelView.updateChannelLogo();
               }
             });
          } else {
             Putpat.trySignup(Settings.getCurrentUser(), function(success) {
                 $(".fehlermeldung").empty().append("Anmeldung fehlgeschlagen. Bitte überprüfe Deine Email-Addresse und Passwort.");
                 ChannelView.updateChannelLogo();
             });
         }
     });
  }

  function tryLogin() {
     $(".fehlermeldung").empty()
     var username = $("#benutzername").val();
     var password = $("#passwort").val();

     if (Utils.isStringEmpty(username)) {
         $(".fehlermeldung").empty().append("Bitte gebe eine Email-Adresse an.");
     } else if (Utils.isStringEmpty(password)) {
         $(".fehlermeldung").empty().append("Bitte gebe ein Passwort an.");
     } else {
         $(".statusmeldung").empty().append("Anmeldung läuft...");

         var user = {
             type: "LOGIN",
             login: username,
             password: password
         }

         loginUser(user);
     }
  }

  function resetButtonState() {
    $("#profile_panel #button_close").attr("src", Positions.size+"images/bt_close.png");

    if (mode == "select_boxes") {
      $("#profile_panel .icons_hover").hide();
      $(".left_box p").css("color", "#000000");
      $(".right_box p").css("color", "#000000");
    } else if (mode == "switch_profile") {
      $("#button_login").attr("src", Positions.size+"images/profil/bt_login.png");
      $("#benutzername").css("border", "2px solid #efefef");
      $("#passwort").css("border", "2px solid #efefef");
     }
  }

  function selectButton(button_name) {
    if (button_name != selected) {
      resetButtonState();
    }

    selected = button_name;

    if (selected == "button_close") {
      $("#profile_panel #button_close").attr("src", Positions.size+"images/bt_close_hover.png");
    } else if (selected == "button_login") {
      $("#button_login").attr("src", Positions.size+"images/profil/bt_login_hover.png");
    } else if (selected == "text_user") {
      $("#benutzername").css("border", "2px solid #00bfff");
    } else if (selected == "text_password") {
      $("#passwort").css("border", "2px solid #00b4ff");
    } else if (selected == "button_left" || selected == "button_right") {
      var box = (selected == "button_left") ? ".left_box" : ".right_box";
      var offset = $(box + " img.img_button").offset();
      var el = $("#profile_panel .icons_hover");
      $(box + " p").css("color", "#3a76a8");
      if ($(box + " img.img_button").width() > 100) {
        el.css("left", (offset.left + Positions.profile.button_offset.x) + "px");
        el.css("top", (offset.top + Positions.profile.button_offset.y)+ "px");
        el.show();
      }
    }
  }

  function moveVertically(dir) {
    if (mode == "select_boxes" || mode == "complete_profile" ||
        (selected == "button_login" && dir > 0)) {
      if (dir > 0 && !standalone) {
        resetButtonState();
        ControlBar.focus();
      }
    } else {
      var newitem= BUTTON_SWITCH_ROTATION.indexOf(selected) + dir;

      if (newitem < 0) {
        newitem = BUTTON_SWITCH_ROTATION.length - 1;
      } else if (newitem >= BUTTON_SWITCH_ROTATION.length) {
        newitem = 0;
      }

      if (standalone && newitem == 0) {
        newitem = 1;
      }

      if (selected == "text_user") {
        userField.unfocus();
      } else if (selected == "text_password") {
        passwordField.unfocus();
      }

      selectButton(BUTTON_SWITCH_ROTATION[newitem]);
    }
  }

  function moveHorizontally(dir) {
    if (mode == "select_boxes") {
      var newitem= BUTTON_ROTATION.indexOf(selected) + dir;

      if (newitem < 0) {
        newitem = BUTTON_ROTATION.length - 1;
      } else if (newitem >= BUTTON_ROTATION.length) {
        newitem = 0;
      }

      selectButton(BUTTON_ROTATION[newitem]);
    }
  }

  function showSelectMenu() {
    mode = "select_boxes";
    selected = "button_left";
    var img = document.createElement('img');
    img.onLoad = img.onload = function() {
      $("#profile_panel .inner_content").empty().append(
           "<img class='icons_bg' src='" + Positions.size + "images/profil/bg_icons.png'/>" +
           "<img class='icons_hover' src='" + Positions.size + "images/profil/icon_hover.png'/>" +
           "<div class='left_box'>"+
              "<h1>Benutzer-Profil vervollständigen</h1>"+
              "<img class='img_button' src='" + Positions.size + "images/profil/bt_tv_to_web.png'/>"+
              "<p>Hier kannst Du Dein Benutzer-Profil vervollständigen und in einen "+
              "vollwertigen Putpat-Benutzer umwandeln.</p>"+
              "<p>Danach kannst du Putpat im Web, auf dem Ipad und TV verwenden.</p>"+
           "</div>"+
           "<div class='right_box'>"+
              "<h1>Benutzer-Profil wechseln</h1>"+
              "<img class='img_button' src='" + Positions.size + "images/profil/bt_change_account.png'/>"+
              "<p>Du bist schon Putpat-Benutzer? Dann kannst Du Dein Profil auf diesem Gerät "+
              "aktivieren.</p>"+
           "</div>"
      );
      selectButton("button_left");
      img = null;

      if (Device.hasMouse) {
        $(".left_box .img_button").mouseover(function() {
          ControlBar.unfocus();
          selectButton("button_left");
        });
        $(".right_box .img_button").mouseover(function() {
          ControlBar.unfocus();
          selectButton("button_right");
        });

        $(".left_box .img_button").click(function() {
          ControlBar.unfocus();
          if (selected != "button_left") {
            selectButton("button_left")
          };
          executeButton("button_left");
        });
        $(".right_box .img_button").click(function() {
          ControlBar.unfocus();
          if (selected != "button_right") {
            selectButton("button_right")
          };
          executeButton("button_right");
        });
      }
    }
    img.src = Positions.size + "images/profil/bt_tv_to_web.png";
  }

  function showCompleteProfile() {
    mode = "complete_profile";
    var pin = Settings.getCurrentUser()["uuid"].substring(0,8);

    $("#profile_panel .inner_content").empty().append(
      "<img class='small_web' src='"+Positions.size + "images/profil/bt_tv_to_web_mini.png'/>"+
      "<div class='small_web_content'>" +
        "<h1>Benutzer-Profil vervollständigen</h1>" +
        "<p>Bitte gehe mit Deinem Computer auf</p>"+
        "<p class='link'>http://www.putpat.tv/pin</p>"+
        "<p>und gib dort diese PIN ein:</p>" +
        "<p class='pin'>PIN: <span class='number'>"+pin+"</span></p>"+
      "</div>"
    );
    selectButton("button_close");
  }

  function checkForPin(pin) {
    var params = {
      "pin": pin
    }
    Putpat.callApi("DeviceLogin.create_login_for_pin", params, function(data) {
      if (data["user"] != null) {
        var user = {
            type: "DEVICE_LOGIN",
            password: data["user"]["password"],
            login: data["user"]["email"]
        }

        loginUser(user);
      } else {
        pinCallTimer = setTimeout(function() {
          checkForPin(pin);
        }, 10000);
      }
    });
  }

  function getUsername() {
    var currentUser = Settings.getCurrentUser();
    if (currentUser != null && (currentUser.type == "LOGIN" || currentUser.type == "DEVICE_LOGIN")) {
      return currentUser.login;
    }
    return "";
  }

  function wireMouseEvents(button) {
    if (Device.hasMouse) {
      $("#"+button).mouseover(function() {
         ControlBar.unfocus();
         selectButton(this.id)
      });
      $("#"+button).click(function() {
        ControlBar.unfocus();
        if (selected != this.id) {
          selectButton(this.id)
        };
        executeButton(this.id);
      });
    }
  }

  function showSwitchProfile() {
    mode = "switch_profile";
    var pin = Utils.createUUID().substring(0,8);

    var title = "Benutzer-Profil wechseln";

    if (standalone) {
      title = panelTitle;
      $("#button_close").hide();
    }

    $("#profile_panel .inner_content").empty().append(
      "<img class='small_profile' src='"+Positions.size + "images/profil/bt_change_account_mini.png'/>"+
      "<div class='small_profile_content'>" +
        "<h1>"+title+"</h1>" +
        "<p>"+panelSubtitle+"</p>"+
        "<div class='variant_left_box'>"+
          "<h2>Variante 1:</h2>"+
          "<p>Melde Dich auf</p>"+
          "<p class='link'>http://www.putpat.tv/tv-login</p>"+
          "<p>mit Deinen Login-Daten an und gib dort diese PIN ein:</p>"+
          "<p class='pin'>PIN: <span class='number'>"+pin+"</span></p>"+
          "<p>Nach ein paar Sekunden wirst Du dann automatisch auf diesem Gerät angemeldet.</p>"+
        "</div>"+
        "<div class='variant_seperator'></div>"+
        "<div class='variant_right_box'>"+
          "<h2>Variante 2:</h2>"+
          "<p>Gib hier Deine Email-Adresse und Passwort ein:</p>"+
          "<p class='input_row'><label>Email:</label> <div style='display:inline-block;position:relative;'><input size='35' type='text' id='benutzername'/><div id='benutzername_click' style='position:absolute; left:0; right:0; top:0; bottom:0;'></div></div></p>"+
          "<p class='input_row'><label>Passwort:</label> <div style='display:inline-block;position:relative;'><input size='35' type='password' id='passwort'/><div id='password_click' style='position:absolute; left:0; right:0; top:0; bottom:0;'></div></div></p>"+
          "<img id='button_login' src='"+Positions.size + "images/profil/bt_login.png'/>"+
          "<p class='fehlermeldung'></p>"+
          "<p class='statusmeldung'></p>"+
        "</div>"+
      "</div>"
    );

    userField = Textinput.init('benutzername',
                            getUsername(),
                            Positions.profile.textinput.x + "px",
                            Positions.profile.textinput.y + "px",
                            userInput, null, function() {
                              $(".variant_left_box").show();
                            }, function() {
                              userFieldInit = true;
                            });

    passwordField = Textinput.init('passwort',
                            "",
                            Positions.profile.textinput.x + "px",
                            Positions.profile.textinput.y + "px",
                            passwordInput, null, function() {
                              $(".variant_left_box").show();
                            }, function() {
                              passwordFieldInit = true;
                            });

    if (Device.hasMouse) {
      $("#benutzername, #benutzername_click").mouseover(function() {
        selectButton("text_user");
      });
      $("#benutzername, #benutzername_click").click(function() {
        executeButton("text_user");
      });
      $("#password, #password_click").mouseover(function() {
        selectButton("text_password");
      });
      $("#password, #password_click").click(function() {
        executeButton("text_password");
      });
    }

    pinCallTimer = setTimeout(function() {
      checkForPin(pin);
    }, 10000);
    selectButton("text_user");
    wireMouseEvents("button_login");
  }

  self.show = function(allGraphics, standaloneMode, title, subtitle) {
    standalone = standaloneMode;
    if (!Utils.isStringEmpty(title)) {
      panelTitle = title;
    }
    if (!Utils.isStringEmpty(subtitle)) {
      panelSubtitle = subtitle;
    } else {
      panelTitle = "";
    }

    ChannelView.hideChannelLogo();

    $("#profile_panel").remove();
    $("#putpat_view").append(
      "<div id='profile_panel'>" +
        "<img class='background' src='" + Positions.size + "images/profil/bg_profil.jpg'/>" +
        "<img id='button_close' class='artist_button' src='"+Positions.size+"images/bt_close.png'/>" +
        "<div class='inner_content'></div>"+
        "<div class='version'>v"+Putpat.VERSION+"</div>"+
      "</div>"
    );

    if (!standalone && !Settings.getCurrentUser().real_account && (Settings.getCurrentUser().uuid != null)) {
      showSelectMenu();
    } else {
      showSwitchProfile();
    }

    self.focus();
    MessageQueue.sendMessage("disable_bauchbinde");
    wireMouseEvents("button_close");
  }

  self.hide = function() {
    if (pinCallTimer != null) {
      clearTimeout(pinCallTimer);
    }
    MessageQueue.sendMessage("enable_bauchbinde");
    $("#profile_panel").hide();
    $("#profile_panel img").remove();
    ChannelView.showChannelLogo();
  }

  self.close = function() {
    if (!standalone) {
      self.hide();
      ChannelView.showChannelLogo();
      ControlBar.currentScreenClosed();
    } else {
      widgetAPI.sendReturnEvent();
    }
  }

  self.focus = function() {
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });
    selectButton(selected);
  }

  self.unfocus = function() {
    resetButtonState();
  }

  return self;
}();
