var VqPanel = function() {
  var self = {};
  var veequalizer = null;
  var sliders = [];
  var selectedSlider = null;
  var selectedButton = null;

  function resetButtonState() {
    if (selectedSlider != null) {
      unselectSlider();
    }
    selectedButton = null;
    $.each(["button_close", "button_los"], function(index, name) {
       $("#"+name).attr('src', $("#"+name).attr('src').replace("_hover.png", ".png"));
    });
  }

  function selectButton(name) {
    resetButtonState();
    selectedButton = name;
    $("#"+name).attr('src', $("#"+name).attr('src').replace(".png", "_hover.png"));
  }

  function executeButton() {
    if (selectedButton == "button_close") {
      self.close();
      Tracking.trackAction("user_action", "vq_cancel");
    } else if (selectedButton == "button_los") {
      Tracking.trackAction("user_action", "vq_start");
      Putpat.updateVeequalizer(veequalizer, function() {
        self.close();
        ControlBar.hide();
        Putpat.getChannels(function(channels) {
           ChannelView.switchToChannel(channels[0], true);
        });
      });
    }
  }

  function nextSlider(position) {
    if (selectedSlider.number() < 4) {
      selectSlider(sliders[selectedSlider.number() + 1], position);
    } else {
      if (position == "decades") {
        selectButton("button_close");
      } else {
        selectButton("button_los");
      }
    }
  }

  function previousSlider(position) {
    if (selectedSlider.number() > 0) {
      selectSlider(sliders[selectedSlider.number() - 1], position);
    } else {
      selectButton("button_los");
    }
  }

  function moveHorizontally(dir) {
    if (selectedButton == "button_close")
      selectSlider(sliders[4], "slider");
    else if (selectedButton == "button_los") {
      selectSlider(sliders[4], "sticker");
    }
  }

  function moveVertically(dir) {
    if (selectedButton == "button_los" && dir == -1) {
      selectButton("button_close");
    } else if (selectedButton == "button_los" && dir == 1) {
      resetButtonState();
      ControlBar.focus();
    } else if (selectedButton == "button_close" && dir == 1) {
      selectButton("button_los");
    }
  }

  function keyboardHandler(event) {
     var keycode = event.keyCode;
     if (keycode == KEYS.KEY_RETURN) {
       self.close();
     } else if (selectedSlider != null) {
       var retVal = selectedSlider.keyboardHandler(event);
       switch (retVal) {
         case "RIGHT_STICKER":
           nextSlider("sticker");
           event.preventDefault();
           break;
         case "LEFT_STICKER":
           previousSlider("sticker");
           event.preventDefault();
           break;
         case "RIGHT_DECADES":
           nextSlider("decades");
           event.preventDefault();
           break;
         case "LEFT_SLIDER":
           previousSlider("slider");
           event.preventDefault();
           break;
         case "DOWN":
           selectButton("button_los");
           event.preventDefault();
           break;
       }
     } else {
       switch(keycode) {
           case KEYS.KEY_LEFT:
               moveHorizontally(-1);
               event.preventDefault();
               break;
           case KEYS.KEY_RIGHT:
               moveHorizontally(+1);
               event.preventDefault();
               break;
           case KEYS.KEY_ENTER:
               executeButton();
               event.preventDefault();
               break;
           case KEYS.KEY_UP:
               moveVertically(-1);
               event.preventDefault();
               break;
           case KEYS.KEY_DOWN:
               moveVertically(1);
               event.preventDefault();
               break;
       }
     }

  }

  function selectSlider(select, position) {
    resetButtonState();
    selectedSlider = select;
    if (position != null) {
      selectedSlider.select(position);
    }
  }

  function unselectSlider() {
    selectedSlider.unselect();
    selectedSlider = null;
  }

  function wireMouseEvents(button) {
    if (Device.hasMouse) {
      $("#"+button).mouseover(function() {
         ControlBar.unfocus();
         selectButton(button)
      });
      $("#"+button).click(function() {
        ControlBar.unfocus();
        if (selectedButton != button) {
          selectButton(button);
        };
        executeButton();
      });
    }
  }

  self.show = function() {
    try {
      Tracking.trackAction("impression", "vq");

      ChannelView.hideChannelLogo();
      $("#vq_panel").remove();
      $("#putpat_view").append(
        "<div id='vq_panel'>"+
           "<img class='background' src='" + Positions.size + "images/vq/bg_vq.jpg'/>" +
        "</div>"
      );

      $("#vq_panel").append(
        "<img id='button_close' class='vq_button' src='"+Positions.size+"images/bt_close.png'/>" + 
        "<img id='vq_logo' src='"+Positions.size+"images/vq/bt_vq_icon.png'/>" +
        "<img id='button_los' src='"+Positions.size+"images/vq/bt_los.png'/>"
      );

      var size = Positions.screen_size[0] + "x" + Positions.screen_size[1];
      if (Putpat.playerAdvert && Putpat.playerAdvert["veequalizer"] && Putpat.playerAdvert["veequalizer"][size]["image"]) {
        $('#vq_panel').append("<img id='veequalizer_branding' src='" +
                  Putpat.playerAdvert["veequalizer"][size]["image"] + "'/>");

        if (Putpat.playerAdvert["veequalizer"][size]["position"]) {
          var el = $("#veequalizer_branding");
          el.css("left", Putpat.playerAdvert["veequalizer"][size]["position"][0] + "px");
          el.css("top", Putpat.playerAdvert["veequalizer"][size]["position"][1] + "px");
        }
        if (Putpat.playerAdvert["veequalizer"]["tracking"].length > 0) {
          Putpat.callUrls(Putpat.playerAdvert["veequalizer"]["tracking"], false, false);
        }
      } else {
        $("#vq_panel").append("<img id='left_info' src='"+Positions.size+"images/vq/banner_help.png'/>");
      }

      sliders = [
        VqSlider(0),
        VqSlider(1),
        VqSlider(2),
        VqSlider(3),
        VqSlider(4)
      ];

      wireMouseEvents("button_close");
      wireMouseEvents("button_los");

      Putpat.getCurrentVeequalizer(function (vq) {
        veequalizer = Utils.clone(vq);
        updateSliders();
      });

      InputHandler.replaceKeyHandlers({
        keyboardHandler: keyboardHandler
      });
      selectSlider(sliders[0], "sticker");
      MessageQueue.sendMessage("disable_bauchbinde");
      MessageQueue.addListener(messageListener);
    } catch(e) {
      Putpat.reportError(e, "unkown", "VqPanel#show", "");
    }
  };

  function messageListener(key, message) {
    if (key == "updateVeequalizer") {
      Putpat.debug("updating vq view!");
      veequalizer = message;
      updateSliders();
    }
  }

  function updateSliders() {
    for(var i = 0;i<5;i++) {
      if (veequalizer.entries[i] == null) {
        veequalizer.entries[i] = {
          'title': null,
          'slider_value':0
        };
      }
      sliders[i].setSlider(veequalizer.entries[i]);
    }
  }

  self.hide = function() {
    try {
      MessageQueue.removeListener(messageListener);
      MessageQueue.sendMessage("enable_bauchbinde");
      $("#vq_panel").hide();
      $("#vq_panel img").remove();
      ChannelView.showChannelLogo();
    } catch(e) {
      Putpat.reportError(e, "unkown", "VqPanel#hide", "");
    }
  };

  self.close = function() {
    try {
      self.hide();
      ControlBar.currentScreenClosed();
    } catch(e) {
      Putpat.reportError(e, "unkown", "VqPanel#close", "");
    }
  };

  self.focus = function() {
    try {
      InputHandler.replaceKeyHandlers({
        keyboardHandler: keyboardHandler
      });
      selectButton("button_los");
    } catch(e) {
      Putpat.reportError(e, "unkown", "VqPanel#focus", "");
    }
  };

  self.unfocus = function() {
    try {
    resetButtonState();
    } catch(e) {
      Putpat.reportError(e, "unkown", "VqPanel#unfocus", "");
    }
  };

  self.setSelectedSlider = function(number) {
    try {
      selectSlider(sliders[number]);
    } catch(e) {
      Putpat.reportError(e, "unkown", "vq_panel#setSelectedSlider", "");
    }
  };

  return self;
}();
